/*
Research Project:   Category Theory Database Tools
                    J. Bradbury, Dr. R. Rosebrugh
                    Mount Allison University 1999
File:               DualCategoryFrame.java
Description:	      This file is displayed when the user selects the "Create
                    Dual Category" menu option in the main window. The user
                    can select a category file(*.CAT) from a choice menu and then
                    enter the name of its dual category in the text field below it.
*/

//import statements
import java.awt.*;
import borland.jbcl.layout.*;
import java.awt.event.*;
import borland.jbcl.control.*;

public class DualCategoryFrame extends Frame
{
  Category c = new Category();
  DefList l = new DefList();
  int check_enter = 0;

  //Constants
  Constants cons = new Constants();
  int MAXWORD = cons.MAXWORD;

  //Objects in the Graphical User Interface(GUI)
  Button exit = new Button();
  Panel button_panel = new Panel();
  BorderLayout borderLayout1 = new BorderLayout();
  XYLayout xYLayout1 = new XYLayout();
  Button okButton = new Button();
  Button cancelButton = new Button();
  Label label2 = new Label();
  Label label1 = new Label();
  TextField nameField = new TextField();
  MainWindow edit = new MainWindow();
  TextField categoryField = new TextField();

  public DualCategoryFrame(MainWindow ed)
  //Default Constructor with the list of categories and the ok
  //button as parameters
  {
    edit = ed;
    try  {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public boolean handleEvent(Event e)
  {
    if ((e.id == Event.WINDOW_DESTROY) || (e.id == Event.ACTION_EVENT))
    {
     hide();            //hide frame
     dispose();         //free resources
     return true;
    }
    else if (e.key == Event.ENTER)
    {
      if (check_enter == 0)
       check_enter++;
      else
      {
       hide();            //hide frame
       dispose();         //free resources
      }
      return true;
    }
    return super.handleEvent(e);
  }

  public boolean action(Event e, Object o)
  {
    if ((e.target == exit) || (e.key == Event.ENTER))
    {
      hide();            //hide frame
      dispose();         //free resources
      return true;
    }
    return false;
  }

  private void jbInit() throws Exception
  //This function intializes the GUI
  {
    this.setTitle("GDCT: Create Dual Category");
    this.setBackground(new Color(192,192,192));
    xYLayout1.setHeight(150);
    okButton.setLabel("OK");
    xYLayout1.setWidth(339);
    okButton.setLabel("OK");
    okButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        okButton_actionPerformed(e);
      }
    });
    cancelButton.setLabel("cancel");
    label2.setText("Current Category:");
    this.setLayout(xYLayout1);
    categoryField.setBackground(Color.white);
    this.add(okButton, new XYConstraints(18, 80, 145, 31));
    this.add(cancelButton, new XYConstraints(171, 79, 145, 31));
    this.add(label2, new XYConstraints(17, 10, -1, -1));
    this.add(label1, new XYConstraints(16, 43, -1, -1));
    this.add(nameField, new XYConstraints(170, 40, 145, 24));
    this.add(categoryField, new XYConstraints(171, 10, 144, 25));
    c = edit.cat;
    nameField.setBackground(Color.white);
    categoryField.setText(c.name);
    categoryField.setEditable(false);
    label1.setText("Dual Category Name:");
  }



  void okButton_actionPerformed(ActionEvent e)
  //Function called when OK button is pressed. This function
  //take the string in the nameField and makes it the name of
  //the current category
  {
    if (nameField.getText().equals(""))
    {
        new MessageDialog(this, "Error", "Dual category name must be filled in before dual category can be created", true);
    }
    else
    {
      makeDual(nameField.getText());
      hide();
      dispose();
    }
  }

  public void makeDual(String name)
  /*
  over, leave:    variables for checking how a person wishes to save their category
  temp2, temp3:   variables for storing temporary paths
  C:      the new category
  */
  {
    char over, leave;
    int i,j,temp;
    int temp2[] = new int [MAXWORD];
    int temp3[] = new int [MAXWORD];
    boolean done;

    Category A = c;
    Category B = new Category();
    B.name = name;

    B.numobj = A.numobj;
    B.numarr = A.numarr;
    B.num_rel = A.num_rel;
    for(i=0;i<A.numobj;i++)
      B.obj[i] = A.obj[i];
    for(i=0;i<A.numarr;i++)
    {
      B.arrow[i] = A.arrow[i];
      B.arr[i][0] = A.arr[i][1];
      B.arr[i][1] = A.arr[i][0];
    }
    for(i=0;i<A.num_rel;i++)
    {
      for(j=path_len(A.rel_set[i].lhs)-1;j>=0;j--)
      {
        temp2[path_len(A.rel_set[i].lhs)-1-j] = A.rel_set[i].lhs[j];
      }
      temp2[path_len(A.rel_set[i].lhs)] = -1;
      for(j=path_len(A.rel_set[i].rhs)-1;j>=0;j--)
      {
        temp3[path_len(A.rel_set[i].rhs)-1-j] = A.rel_set[i].rhs[j];
      }
      temp3[path_len(A.rel_set[i].rhs)] = -1;
      Relation r = new Relation();
      if(greater(temp2,temp3))
      {
        r.lhs = copy_path(r.lhs,temp2);
        r.rhs = copy_path(r.rhs,temp3);
      }
      else
      {
        r.lhs = copy_path(r.lhs,temp3);
        r.rhs = copy_path(r.rhs,temp2);
      }
      B.rel_set[i] = r;
    }
    edit.List.insertNode(B);  //put category in Linked List
    edit.cat = B;
    edit.textCardLayout.show(edit.textPanel, "displayPanel");
    //Clear graphical display HERE
    edit.categoryChoice.add(edit.cat.name);
    edit.categoryChoice.select(edit.cat.name);
    this.setTitle("Category Theory Database Tools [ CTDT 1999 ]");
    validate();
    edit.displayCategory(edit.cat);
    edit.GraphCategory(edit.graph_, edit.graphCanvas_, edit.cat);
  }

  public int path_len(int path[])
  /*
  Parameters:
        path:   an array of integers terminated by -1 which is a path
  Purpose:
        To determine the length of an array of integers terminated by -1
  Design:
        Returns the number of integers in the path, does not count -1
  */
  {
    int length = 0;
    while (path[length] != -1)
    {
      length++;
    }
    return(length);
  }


 public boolean greater(int path1[], int path2[])
 /*
 Parameters:
        path1:  an array of integers terminated by -1
        path2:  an array of integers terminated by -1
 Purpose:
        To determine if path1 has a greater ordering than path2
 Design:
        Returns true if path1 is of greater order than path2
 */
 {
  int j;

        if(path_len(path1) > path_len(path2))
        {
        return(true);
        }
        else
        {
                if ((path_len(path1)) == (path_len(path2)))
                {
                        j = 0;
                        while  (j < path_len(path1))
                        {
                                if (path2[j] > path1[j])
                                {
                                        return(true);
                                }
                                else
                                {
                                        if (path2[j] < path1[j])
                                        {
                                                return(false);
                                        }
                                }
                                j++;
                        }
                }
        }
        return(false);
 }

   public int[] copy_path(int dest[], int source[])
  /*
  Parameters:
        dest:   an array of integers which will be set to equal source
        source: an array of integers which is copied into dest
  Purpose:
        To copy one array of integers to another array of integers
  Design:
        Copies source into dest
  */
  {
    int i;
    int temp[] = new int [MAXWORD];

    for(i=0;i<=path_len(source);i++)
    {
      temp[i] = source[i];
    }
    return(temp);
  }



}