/*
Research Project:   Category Theory Database Tools
                    J. Bradbury, Dr. R. Rosebrugh
                    Mount Allison University 1999
File:               MakeConfluentFrame.java
Description:	      This file is displayed when the user selects the "Make
                    Confluent " menu option in the main window. The user
                    can select a category file(*.CAT) from a choice menu and then
                    changes made to the category to make it confluent will be
                    displayed in the text area. To save the changes to the current
                    category press "ok".
*/

//import statements
import java.awt.*;
import borland.jbcl.layout.*;
import java.awt.event.*;
import borland.jbcl.control.*;

public class MakeConfluentFrame extends Frame
{
  Category cat = new Category();
  DefList l = new DefList();
  int check_enter = 0;

  //Constants
  Constants cons = new Constants();
  int MAXWORD = cons.MAXWORD;

  //Objects in the Graphical User Interface(GUI)
  Button exit = new Button();
  Panel button_panel = new Panel();
  BorderLayout borderLayout1 = new BorderLayout();
  XYLayout xYLayout1 = new XYLayout();
  Button okButton = new Button();
  Button cancelButton = new Button();
  Label label2 = new Label();
  TextArea textArea1 = new TextArea("", 10, 10, TextArea.SCROLLBARS_VERTICAL_ONLY);
  MainWindow edit = new MainWindow();
  TextField categoryField = new TextField();

  public MakeConfluentFrame(MainWindow ed)
  //Default Constructor with the list of categories and the ok
  //button as parameters
  {
    edit = ed;
    try  {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public boolean handleEvent(Event e)
  {
    if ((e.id == Event.WINDOW_DESTROY) || (e.id == Event.ACTION_EVENT))
    {
     hide();            //hide frame
     dispose();         //free resources
     return true;
    }
    else if (e.key == Event.ENTER)
    {
      if (check_enter == 0)
       check_enter++;
      else
      {
       hide();            //hide frame
       dispose();         //free resources
      }
      return true;
    }
    return super.handleEvent(e);
  }

  public boolean action(Event e, Object o)
  {
    if ((e.target == exit) || (e.key == Event.ENTER))
    {
      hide();            //hide frame
      dispose();         //free resources
      return true;
    }
    return false;
  }

  private void jbInit() throws Exception
  //This function intializes the GUI
  {
    this.setTitle("GDCT: Make Category Confluent");
    this.setBackground(new Color(192,192,192));
    xYLayout1.setHeight(210);
    okButton.setLabel("OK");
    xYLayout1.setWidth(339);
    okButton.setLabel("OK");
    okButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        okButton_actionPerformed(e);
      }
    });
    cancelButton.setLabel("cancel");
    label2.setText("Current Category:");
    this.setLayout(xYLayout1);
    categoryField.setBackground(Color.white);
    this.add(okButton, new XYConstraints(19, 144, 145, 31));
    this.add(cancelButton, new XYConstraints(171, 144, 145, 31));
    this.add(label2, new XYConstraints(17, 10, -1, -1));
    this.add(textArea1, new XYConstraints(21, 40, 295, 98));
    this.add(categoryField, new XYConstraints(163, 7, 152, 25));
    cat = edit.cat;
    categoryField.setText(cat.name);
    categoryField.setEditable(false);
    textArea1.setBackground(Color.white);
    make_confluent();    //Call confluence
  }

  void okButton_actionPerformed(ActionEvent e)
  //Function called when OK button is pressed. This function
  //take the string in the nameField and makes it the name of
  //the current category
  {
    edit.List.replaceNode(cat);
    edit.cat = cat;
    //edit.displayArea.setText("");
    //Clear graphical display HERE
    edit.displayCategory(edit.cat);
    edit.GraphCategory(edit.graph_, edit.graphCanvas_, edit.cat);
    hide();
    dispose();
  }

  public int overlap(int path1[], int path2[])
  /*
  Parameters:
        path1, path2: arrays of integers representing two paths of arrows.
  Purpose:
        To determine if there is an overlap between the two paths and to
        return its length; if there is no overlap, returns -1.
  Design:
        Compare the beginning of path1 to the end of path2.  If they are
        the same, return the size of the overlap; otherwise return -1.
  */
  {
    int i, k;
    //the length of the smaller of path1 and path2
    int size =0;
    //temporary variables to store portions of path1 and path2
    int temp1[] = new int [MAXWORD];
    int temp2[] = new int [MAXWORD];

    if(path_len(path1) > path_len(path2))
    {
      size = path_len(path2);
    }
    else
    {
      size = path_len(path1);
    }
    for(i=0;i<size;i++)
    {
      temp1 = copy_path(temp1,path1);
      temp1[size-i] = -1;
      for(k=0;k<size-i;k++)
      {
        int indexof = path_len(path2) - size + k +i;
        temp2[k] = path2[indexof];
      }
      temp2[k] = -1;
      if(equals(temp1,temp2))
      {
        return(size-i);
      }
    }
    return(-1);
  }

  public void add_relation(int w1[], int w2[])
  /*
  Parameters:
        w1,w2:  arrays of integers representing paths which are the left and
                        right sides of a relation to be added to the category.
        A:      pointer to the current category.
  Purpose:
        To add the relation w1 = w2 to the set of relations in the category *A.
  Design:
        The number of relations in the category is incremented.  Whichever of
        w1 and w2 has the greater order is stored as the left side of the new
        relation, and the other as the right side.
  */
  {
    Relation r = new Relation();

    cat.num_rel++;

    if (greater(w1,w2))
    {
      r.lhs = copy_path(r.lhs,w1);
      r.rhs = copy_path(r.rhs,w2);
    }
    else
    {
      r.lhs = copy_path(r.lhs,w2);
      r.rhs = copy_path(r.rhs,w1);
    }
    cat.rel_set[cat.num_rel-1] = r;

    int i = 0;
    while (cat.rel_set[cat.num_rel-1].lhs[i] != -1)
    {
      textArea1.append("cat.rel_set[cat.num_rel-1].lhs[" + i + "] = " + cat.rel_set[cat.num_rel-1].lhs[i] + "\n");
      i++;
    }
    i = 0;
    while (cat.rel_set[cat.num_rel-1].rhs[i] != -1)
    {
      textArea1.append("cat.rel_set[cat.num_rel-1].rhs[" + i + "] = " + cat.rel_set[cat.num_rel-1].rhs[i] + "\n");
      i++;
    }

    //Display  on the screen relation that is being added to category
    textArea1.append("\nThe relation " + path_to_string(cat.rel_set[cat.num_rel-1].lhs, cat));
    textArea1.append(" = " + path_to_string(cat.rel_set[cat.num_rel-1].rhs, cat) + " has been added to the set of relations.");
  }

  public boolean reduce_overlap(Relation rel1, Relation rel2, int olap_size)
  /*
  Parameters:
        rel1, rel2:     two relations from the current category whose left sides
                                have an overlap.
        olap_size:  the size of the overlap between rel1 and rel2.
        A:                      pointer to the current category.
  Purpose:
        To check to see that rel1 and rel2 satisfy the Overlap rule of the
        Knuth-Bendix algorithm and if not, to add a relation to the set
        of relations in the category.  Returns a boolean indicating whether
        or not a relation was added.
  Design:
        Construct r1 and r2 as described below in the variable declarations.
        If w1 and w2 (the reduced forms of r1 and r2) are not equal, then
        add the relation w1 = w2 to the category.
  */
  {
    //arrows in left side of rel1 BEFORE the overlap, plus right side of rel2
    int r1[] = new int [MAXWORD];
    // right side of rel1, plus arrows in left side of rel2 AFTER the overlap
    int r2[] = new int [MAXWORD];
    //reduced forms of r1 and r2
    int w1[] = new int [MAXWORD];
    int w2[] = new int [MAXWORD];
    //temporary variable to store path
    int temp[] = new int[MAXWORD];

    r2 = copy_path(r2,rel1.rhs);
    temp = copy_sub(temp, rel2.lhs, olap_size, path_len(rel2.lhs)-olap_size);
    r2 = append_path(r2,temp);
    r1 = copy_sub(r1, rel1.lhs, 0, path_len(rel1.lhs) - olap_size);
    r1 = append_path(r1, rel2.rhs);
    w1 = reduce(w1, r1, cat);
    w2 = reduce(w2, r2, cat);

    if (!(equals(w1,w2)))
    {
      add_relation(w1,w2);
      return(true);
    }
    return(false);
  }

  public boolean reduce_subpath(Relation rel1, Relation rel2, int location)
  /*
  Parameters:
        rel1, rel2:     two relations from the current category where the left
                                side of rel2 is a subpath of the left side of rel1.
        location:       the location of rel2 inside rel1.
        A:                      pointer to the current category.
  Purpose:
        To check to see that rel1 and rel2 satisfy the Subpath rule of the
        Knuth-Bendix algorithm and if not, to add a relation to the set of
        relations in the category.  Returns a boolean indicating whether
        or not a relation was added.
  Design:
        Construct r1 and r2 as described below in the variable declarations.
        If w1 and w2 (the reduced forms of r1 and r2) are not equal, then
        add the relation w1 = w2 to the category.
  */
  {
    //right side of rel1
    int r1[] = new int [MAXWORD];
    //left side of rel1 with left side of rel2
    //(substring) replaced by right side of rel2
    int r2[] = new int [MAXWORD];
    //reduced forms of r1 and r2
    int w1[] = new int [MAXWORD];
    int w2[] = new int [MAXWORD];
    //temporary variable to store path
    int temp[] = new int [MAXWORD];

    r1 = copy_path(r1,rel1.rhs);
    copy_sub(r2,rel1.lhs,0,location);
    append_path(r2,rel2.rhs);
    copy_sub(temp,rel1.lhs,location+path_len(rel2.lhs),path_len(rel1.lhs)-(location+path_len(rel2.lhs)));
    append_path(r2,temp);
    w1 = reduce(w1,r1, cat);
    w2 = reduce(w2,r2, cat);

    if (!equals(w1,w2))
    {
      add_relation(w1,w2);
      return(true);
    }
    return(false);
  }

  public void make_confluent()
  /*
  Parameter:
        A:              pointer to the current category
  Purpose:
        To check all pairs of relations in the category for the Overlap and
        Subpath rules and to add relations to make the set of relations confluent.
  Design:
        Using a double for loop, compare all pairs of relations.  Check for
        any overlaps between the left sides of the two relations, and call
        function reduce_overlap to add any relations necessary to make the
        set of relations confluent.  Then, check if the left side of one
        relation is a subpath of the other left side, and call function
        reduce_subpath to add any necessary relations to the category.
  */
  {
    //set to true when all overlaps and subpaths have been checked,
    //and the set of relations is confluent
    boolean done;
    //set to true when all overlaps have been checked
    boolean finished;

    int i,j;
    //size of an overlap between two paths
    int size;
    //location of one path inside another
    int loc;
    //position in one path to begin searching for another path as a substring
    int position;
    //temporary variables to store paths
    int temp1[] = new int [MAXWORD];
    int temp2[] = new int [MAXWORD];

    done = false;
    while(!done)
    {
      done = true;
      //viewbox.appendText("cat.num_rel = " + cat.num_rel);
      for(i=0;i<cat.num_rel;i++)
      {
        //viewbox.appendText("\ni =" + i);
        for(j=0;j<cat.num_rel;j++)
        {
          //viewbox.appendText("\nj=" + j);
          int t[] = new int[30];
          int p[] = new int[30];
          t = cat.rel_set[j].lhs;
          p = cat.rel_set[i].lhs;
          if((size = overlap(t,p)) != -1)
          {
            //textArea1.appendText("IMP SIZE = " + size);
            //textArea1.appendText("olap done");
            finished = false;
            while(!finished)
            {
              //viewbox.appendText("while!(finished)");
              //viewbox.appendText("i = " + i + "j = " + j);
              Relation t1 = cat.rel_set[i];
              Relation t2 = cat.rel_set[j];
              if(reduce_overlap(t1,t2,size))
              {
                //viewbox.appendText("\ndone = false");
                done = false;
              }
              //viewbox.appendText("size =" + size);
              if(size == 1)
              {
                //viewbox.appendText("\nfinished = true");
                finished = true;
              }
              else
              {
                //viewbox.appendText("make con");
                size--;
                copy_sub(temp1,cat.rel_set[i].lhs,(path_len(cat.rel_set[i].lhs) - size),size);
                copy_sub(temp2,cat.rel_set[j].lhs,0,size);
                if(!equals(temp1,temp2))
                {
                  //viewbox.appendText("\nfinished = true2");
                  finished = true;
                }
              }
            }
          }
          //workbox.getText();
          position = 0;
          while((loc = search(cat.rel_set[i].lhs,cat.rel_set[j].lhs,position)) != -1)
          {
            position = loc + 1;
            if(reduce_subpath(cat.rel_set[j],cat.rel_set[i],loc))
            {
              done = false;
            }
          }
        }
      }
    }
    if (textArea1.getText().equals(""))
      textArea1.appendText("No relations added to category");
    textArea1.appendText("\nThe set of reductions is now confluent...");
  }

  public boolean equals(int path1[], int path2[])
  /*
  Parameters:
        path1:  an array of integers terminated by -1 which is a path
        path2:  an array of integers terminated by -1 which is a path
  Purpose:
        To determine if two paths are identical
  Design:
        Returns true if path1 is identical to path2 and false otherwise
                note:this function does not reduce them at all
  */
  {
    int i;

    //Remove identity arrows in path1 if path1 != "1" itself
    if ((path_len(path1) != 1))
    {
      i  = 0;
      while (path1[i] != -1)
      {
        if (path1[i] == cons.id_code)
          for (int j = i; j < (path_len(path1)); j++)
            path1[j] = path1[j+1];
        i++;
      }
    }
    //Remove identity arrows in path2 if path2 != "1" itself
    if ((path_len(path2) != 1))
    {
      i = 0;
      while (path2[i] != -1)
      {
        if (path2[i] == cons.id_code)
          for (int j = i; j < (path_len(path2)); j++)
            path2[j] = path2[j+1];
        i++;
      }
    }

    i = 0;
    if((path_len(path1)) == (path_len(path2)))
    {
      for(i=0;i<path_len(path1);i++)
      {
        if(path1[i] != path2[i])
        {
          return(false);
        }
      }
      return(true);
    }
    else
    {
      return(false);
    }
  }

  public int search(int sub[], int target[], int start)
  /*
  Parameters:
        sub:    an array of integers which is the subpath we are looking for
        target: an array of integers which is being searched for sub
        start:  The place in target where the search is to begin
  Purpose:
        To search one path, begining at some starting position, to see if another
        path is a subpath
  Design:
        Returns the position where the subpath begins in the path being searched,
         and -1 if sub is not a subpath of target
  */
  {
    int i,j;
    boolean found;

    found = false;
    for (i=start;i<path_len(target);i++)
    {
      if (sub[0] == target[i])
      {
        found = true;
        for (j=0;j<path_len(sub);j++)
        {
          if (target[i+j] != sub[j])
          {
            found = false;
          }
        }
      }
      if (found)
      {
        break;
      }
    }
    if (found)
    {
      return(i);
    }
    else
    {
      return(-1);
    }
  }

  public int[] copy_sub(int dest[], int source[], int start, int length)
  /*
  Parameters:
        dest:   a substring of source is copied into dest
        source: a substring of source is copied into dest
        start:  the position in source at which the substring starts
        length: the length of the substring
  Purpose:
        To copy a substring of one array into another array
  Design:
          Makes dest equal a substring of source
  */
  {
    int i;

    for(i=0;i<length;i++)
    {
      dest[i] = source[start+i];
    }
    dest[i] = -1;
    return(dest);
  }

  public int[] reduce(int reducedpath[], int path[], Category c)
  /*
  Parameters:
        reducedpath:    an array of integers which will be set to equal the
                                        reduced path
        path:                   the array of integers being reduced
        A       :                       a category
  Purpose:
        To reduce a path using the set of relations stored in a category
  Design:
        Reduces path and stores its reduced form in reducedpath
  */
  {
    int i,j;
    //the spot where the left side of a relation
    //begins in the path being reduced
    int spot;
    //a temporary variable used to store the length of a path
    int temp;
    //the difference in length between the left side of a relation and the
    //right side of the relation
    int diff;

    reducedpath = copy_path(reducedpath,path);
    for (i=0;i<c.num_rel;i++)
    {
      if((spot = search(c.rel_set[i].lhs,reducedpath,0)) != -1)
      {
        for (j=spot;j<spot+path_len(c.rel_set[i].rhs);j++)
        {
          reducedpath[j] = c.rel_set[i].rhs[j - spot];
        }
        //viewbox.appendText("reducedpath[i] = " + reducedpath[i]  + "\n");
        temp = path_len(reducedpath);
        diff = path_len(c.rel_set[i].lhs) - path_len(c.rel_set[i].rhs);
        for (j=spot+path_len(c.rel_set[i].lhs);j<temp;j++)
        {
          reducedpath[j - diff] = reducedpath[j];
        }
        reducedpath[j - diff] = -1;
        i = -1;
      }
    }
    return(reducedpath);
  }

  public int[] append_path(int dest[], int source[])
  /*
  Parameters:
        dest:   an array of integers is appended to this array of integers
        source: this array of integers is appended to dest
  Purpose:
        To append one array of intgers to another array of integers, all terminated
        by -1
  Design:
        Appends source to dest
  */
  {
    int i, size;

    size = path_len(dest);
    for(i=0;i<=path_len(source);i++)
    {
      dest[size+i] = source[i];
    }
    return (dest);
  }

  public String path_to_string(int path[], Category A)
  /*
  Parameters:
        path:   the path to be converted to a string(function does not change
                         path)
        A:              a category that contains the arrows of the path
  Purpose:
        To convert an array of integers, which represent arrows, terminated by -1
        into a path which is a string of characters which are arrows and the string
        is terminated by \0.
  Design:
        Returns a string which is the path represented by the integers in the
        integer array
  */
  {
    int i;
    String temp = new String();

    if ((path[0] == cons.id_code))
    {
      return("1");
    }
    else
    {
      for(i=0;i<path_len(path);i++)
      {
        if (i < path_len(path)-1)
  		    temp = temp + A.arrow[path[i]] + "*";
        else
  		    temp = temp + A.arrow[path[i]];
      }
      temp = temp + '\0';
      return(temp);
    }
    //return ("NOT 1");
  }


  public int path_len(int path[])
  /*
  Parameters:
        path:   an array of integers terminated by -1 which is a path
  Purpose:
        To determine the length of an array of integers terminated by -1
  Design:
        Returns the number of integers in the path, does not count -1
  */
  {
    //length of path
    int length = 0;

    while (path[length] != -1)
    {
      length++;
    }
    return(length);
  }

  public int[] copy_path(int dest[], int source[])
  /*
  Parameters:
        dest:   an array of integers which will be set to equal source
        source: an array of integers which is copied into dest
  Purpose:
         To copy one array of integers to another array of integers
  Design:
        Copies source into dest
  */
  {
    int i;
    int temp[] = new int [MAXWORD];

    for(i=0;i<=path_len(source);i++)
    {
      temp[i] = source[i];
    }
    return(temp);
  }

  public boolean greater(int path1[], int path2[])
  /*
  Parameters:
        path1:  an array of integers terminated by -1
        path2:  an array of integers terminated by -1
  Purpose:
        To determine if path1 has a greater ordering than path2
  Design:
        Returns true if path1 is of greater order than path2
  */
  {
    int j;

    if(path_len(path1) > path_len(path2))
    {
      return(true);
    }
    else
    {
      if ((path_len(path1)) == (path_len(path2)))
      {
      if ((path_len(path1) == 1) && (path1[0] == cons.id_code))
        return(false);
      j = 0;
      while  (j < path_len(path1))
      {
        if (path2[j] > path1[j])
        {
          return(false);
        }
        else
        {
          if (path2[j] < path1[j])
          {
            return(true);
          }
        }
        j++;
        }
      }
    }
    return(false);
  }
}
