 /*
Research Project:   Category Theory Database Tools
                    J. Bradbury, Dr. R. Rosebrugh
                    Mount Allison University 1999
File:               printCategory.java
Description:        This class is implemented to print a text version or a
                    graphical version of the current category.
*/

//import statements
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class printCategory extends Frame
{

  MainWindow ed = new MainWindow();
  Category c = new Category();
  int x = 50;
  int y = 50;
  int inc = 15;
  private Matrix44	rotxMatrix_, rotzMatrix_;

  public printCategory(MainWindow e)
  //Default constructor
  {
    ed = e;
    c = ed.cat;
  }

  public void printTextCategory()
  //Print text version of current category
  {
    PrintJob pjob;
    Properties p = new Properties();
    Graphics pgraphics;
    String name = new String("Print Category(Text)");
    pjob = getToolkit().getPrintJob(this, name, p);
    if(pjob != null)
    {
      pgraphics = pjob.getGraphics();
      if(pgraphics != null)
      {
        pgraphics.setFont(new Font("Serif",Font.PLAIN,14));
        printOAR(pgraphics);
        pgraphics.dispose();
      }
      pjob.end();
    }
  }

  public void printGraphicalCategory()
  //Print graphical version of current category
  {
    PrintJob pjob;
    Properties p = new Properties();
    Graphics pgraphics;
    String name = new String("Print Category(Graphical)");
    pjob = getToolkit().getPrintJob(this, name, p);
    if(pjob != null)
    {
      pgraphics = pjob.getGraphics();
      if(pgraphics != null)
      {
        pgraphics.setFont(new Font("Serif",Font.PLAIN,14));
        printAxes(pgraphics);
        printObjects(false, pgraphics, 0);
        printArrows(pgraphics);
        pgraphics.dispose();
      }
      pjob.end();
    }
  }

  public void printOAR(Graphics g)
  //Prints text version of category
  {
    int counter = 0;

    //Print Category Name
    String cat = "category";
    g.drawString(cat, x, y);
    y = y + inc;
    g.drawString("         " + c.name, x, y);
    y = y + inc;

    //Print Objects
    g.drawString("objects", x, y);
    String objects = "         ";
    for (int i = 0; i < c.numobj; i++)
    {
      if (i < (c.numobj - 1))
         objects = objects.concat(c.obj[i] + ", ");
      if (i == (c.numobj - 1))
         objects = objects.concat(c.obj[i] + ".\n");
    }
    y = y + inc;
    g.drawString(objects, x, y);
    y = y + inc;

    //Print Arrows
    g.drawString("arrows", x, y);
    String arrows = "         ";
    for (int i = 0; i < c.numarr; i++)
    {
      if (i < (c.numarr - 1))
      {
         arrows = arrows.concat("         " + c.arrow[i] + ":" + c.obj[c.arr[i][0]] + "->" + c.obj[c.arr[i][1]] + ", ");
      }
      if (i == (c.numarr - 1))
      {
        arrows = arrows.concat("         " + c.arrow[i] + ":" + c.obj[c.arr[i][0]] + "->" + c.obj[c.arr[i][1]] + ". ");
      }
      counter++;
      if ((counter == 4) || (i == (c.numarr-1)))
      {
          counter = 0;
          y = y + inc;
          g.drawString(arrows, x, y);
          arrows = new String();
      }
    }
    y = y + inc;

    //Print Relations
    g.drawString("relations", x, y);
    printRelations(g);
  }

  public void printRelations(Graphics g)
  //prints relations of current category to the printer
  {
      int i,j;
      int counter = 0;
      String relation = new String();
      relation = "         ";
      for (i=0;i<c.num_rel-1;i++)
      {
        j = 0;
        while (c.rel_set[i].lhs[j] != -1)
        {
          relation =  relation.concat("" + c.arrow[c.rel_set[i].lhs[j]]);
          j++;
        }
        relation =  relation.concat(" = ");
        j = 0;
        //Display right side of relation
        if (c.rel_set[i].rhs[0] == ed.cons.id_code)
          relation =  relation.concat("1");
        else
        {
          while (c.rel_set[i].rhs[j] != -1)
          {
            relation =  relation.concat("" + c.arrow[c.rel_set[i].rhs[j]]);
            j++;
          }
        }
        counter++;
        relation =  relation.concat(",");
        if (counter == 4)
        {
          counter = 0;
          if (i <  c.num_rel-2)
          {
            y = y + inc;
            g.drawString(relation, x, y);
            relation = "         ";
          }
        }
      }
      if (c.num_rel != 0)
      {
        j = 0;
        while (c.rel_set[i].lhs[j] != -1)
        {
          relation =  relation.concat("" +c.arrow[c.rel_set[i].lhs[j]]);
          j++;
        }
        relation =  relation.concat(" = ");
        j = 0;
        if (c.rel_set[i].rhs[0] == ed.cons.id_code)
          relation =  relation.concat("1");
        else
        {
          while (c.rel_set[i].rhs[j] != -1)
          {
            relation =  relation.concat("" +c.arrow[c.rel_set[i].rhs[j]]);
            j++;
          }
        }
        relation =  relation.concat(".\n");
        y = y + inc;
        g.drawString(relation, x, y);
      }
      else
      {
        relation =  relation.concat(".\n");
        y = y + inc;
        g.drawString(relation, x, y);
      }
  }


   private synchronized void printAxes(Graphics graphics)
   //Print 3D axes in top right corner of page
   {
        String letter;
        double lx, ly;
        rotxMatrix_ = ed.graphCanvas_.rotxMatrix_;
        rotzMatrix_ = ed.graphCanvas_.rotzMatrix_;
        FontMetrics fm = graphics.getFontMetrics();

        graphics.setColor(Color.black);

        DPoint3 p2 = new DPoint3();
        p2.move(25, 0, 0);
        p2.transform(rotzMatrix_);
        p2.transform(rotxMatrix_);
        graphics.drawLine(40, 40, 40 + (int)p2.x, 40 + (int)p2.y);

        p2.move(32, 0, 0);
        p2.transform(rotzMatrix_);
        p2.transform(rotxMatrix_);
        letter = new String("X");
        lx = p2.x - fm.stringWidth(letter) / 2.0;
        ly = p2.y + fm.getAscent() / 2.0;
        graphics.drawString(letter, 40 + (int)lx, 40 + (int)ly);

        p2.move(0, 25, 0);
        p2.transform(rotzMatrix_);
        p2.transform(rotxMatrix_);
        graphics.drawLine(40, 40, 40 + (int)p2.x, 40 + (int)p2.y);

        p2.move(0, 32, 0);
        p2.transform(rotzMatrix_);
        p2.transform(rotxMatrix_);
        letter = new String("Y");
        lx = p2.x - fm.stringWidth(letter) / 2.0;
        ly = p2.y + fm.getAscent() / 2.0;
        graphics.drawString(letter, 40 + (int)lx, 40 + (int)ly);

        p2.move(0, 0, 25);
        p2.transform(rotzMatrix_);
        p2.transform(rotxMatrix_);
        graphics.drawLine(40, 40, 40 + (int)p2.x, 40 + (int)p2.y);

        p2.move(0, 0, 32);
        p2.transform(rotzMatrix_);
        p2.transform(rotxMatrix_);
        letter = new String("Z");
        lx = p2.x - fm.stringWidth(letter) / 2.0;
        ly = p2.y + fm.getAscent() / 2.0;
        graphics.drawString(letter, 40 + (int)lx, 40 + (int)ly);
  }

  public synchronized void printObjects(boolean selected, Graphics graphics, int which_gr)
  //Print objects grpahical
  {
    Node tmpnode;

    for(tmpnode = ed.graph_.firstNode(); tmpnode != null; tmpnode = ed.graph_.nextNode(tmpnode))
      if(tmpnode.getSelected() == selected)
        tmpnode.draw(this, graphics, ed.graphCanvas_.viewTransform_, ed.graphCanvas_.quality_, Color.black);
  }

  public void printArrows(Graphics g)
  //Print arrows graphical
  {
    Enumeration edges = ed.graph_.getEdges();
    boolean directed = ed.graph_.isDirected();

    while(edges.hasMoreElements())
    {
      Edge edge = (Edge)(edges.nextElement());
      if (drawOpp(edge))
        edge.draw(g, ed.graphCanvas_.viewTransform_, ed.graphCanvas_.xyPlane_, true, false, ed.graphCanvas_.quality_, ed.graphCanvas_, 1, Color.black, true, null);
    }
  }

    public boolean drawOpp(Edge curredge)
  {
    boolean curr = false;
    boolean opp = false;
    Enumeration eds = ed.graph_.getEdges();

    while(eds.hasMoreElements())
    {
      Edge edge = (Edge)(eds.nextElement());
      if ((edge.head_.getLabel().equals(curredge.tail_.getLabel())) && (edge.tail_.getLabel().equals(curredge.head_.getLabel())))
        opp = true;
      if ((edge.tail_.getLabel().equals(curredge.tail_.getLabel())) && (edge.head_.getLabel().equals(curredge.head_.getLabel())))
        curr = true;
      if (curr == true)
        return (true);
      if (opp == true)
        return (false);
    }
    return (true);
  }

}
