//import statements
import java.awt.*;
import java.applet.Applet;

class tree
{
/*
Parameters:
	None:
Purpose:
	To create the first node of a tree.
Design:
	Makes the first node of a tree with a * as the data.
*/

nodetype previous = new nodetype();

public nodetype create_tree(TextArea viewbox)
{
  nodetype temp = new nodetype();
	temp.data = -1;
 // viewbox.appendText("data = -1\n");
	temp.down = null;
	temp.across = temp; // the across node is itself on initialization
	temp.tree = null;
	temp.flag = false; // set false here, made true when a right sibling exists
	temp.info = 0;
	return(temp);
}

/*
Parameters:
	currentnode: a node to which we wish to add a child with contents `info'.
	info:       the data we wish to put in the new node.
Purpose:
	To insert a node into a tree with an alphabetical ordering.
Design:
	Make a new node which we insert as the first child of the node
	by currentnode.  Then sort the children of currentnode
  in alphabetical order.  Returns the inserted node.
*/

public nodetype add_node(nodetype currentnode, int info, Category A, TextArea viewbox)
{
  nodetype temp = new nodetype();  /* the new node being created */
  nodetype temp2 = new nodetype(); /* used during the sort */

	temp.flag = (currentnode.down != null);
	if (temp.flag)
		{
		temp.across = currentnode.down;    /*link node to other children*/
		}
	else
		{
		temp.across = currentnode;   /* new node is only child, across link is up */
		}
	currentnode.down = temp;            /*link node to parent*/
	while((temp.flag) && (temp.across.data < info))
  {  /*sort alphabetically, temp.across migrates left as needed*/
		temp.data = temp.across.data;
		temp.down = temp.across.down;
		temp.info = temp.across.info;
		if (temp.across.tree != temp.across)
			{
			temp.tree = temp.across.tree;
			}
		else
			{
			temp.tree = temp;
			}
		if (temp.down != null)
			{
			temp2 = temp.down;
			while(temp2.flag)
				{
				temp2 = temp2.across;
				}
			temp2.across = temp;
			}
		temp = temp.across;
	}
	temp.data = info;              /*insert data into new node*/
	temp.tree = null;
	temp.info = 0;
	temp.down = null;
	return(temp);
}

/*
Parameters:
	node:   a node on the level of the tree that will be searched
	info:   the data that is to be searched for
Purpose:
	To search a level of a tree for a node containing the data info.
Design:
	To search the entire level of the tree the	first node of the
  level should be passed in.  If returns true, then
	node contains info,  otherwise node is the last node of the level.
*/

public boolean search_level(nodetype node, int info)
{
	while((node.flag) && (node.data != info))
  {
		node = node.across;  /*if data not found and not at*/
	}                     /*this node of level, move across*/
	if(node.data == info)
	{
		 return(true);
	}
	else
	{
		return(false);       // node is last on the level
	}
}

/*
Parameters:
	tree: the tree to be searched
	list: the list to be searched for
	pointer: returns a pointer to the last node of the list in the tree
Purpose:
	Searches a tree for the list and returns true if it is found and false otherwise
Design:
	Returns true if list found in tree and false otherwise.  Sets what pointer points to
	to point to last node of list in tree if found and sets what pointer points to to null
	if not found.
*/

public boolean search_tree(nodetype tree, listnodetype list, nodetype pointer)
{
 boolean finished;
 nodetype nodepointer;
 listnodetype temppointer;

	finished = false;
	temppointer = list;
	/*if tree is empty, set nodepointer to point to top of tree and do not search
		tree*/

	if ((nodepointer = tree.down) == null)
		{
		finished = true;
		nodepointer = tree;
		}

	/*search for list until list is found or some element of list not in tree*/

	while((temppointer != null) && (!finished))
		{

		/*      if current element of list not found in proper level of tree
			then exit while loop and have nodepointer point to parent of level
			just searched */

		if(!search_level(nodepointer,temppointer.data))
			{
			finished = true;
			nodepointer = nodepointer.across;
			}

		/*      if current element of list is in tree and not last element of
			list and at the end of a branch of tree then exit while loop.
			(list not in tree) */

		else
			{
			if((temppointer.next != null) && (nodepointer.down == null))
				{
				finished = true;
				}

		  /*    if current element of list in tree, and not at end of branch
				of tree then move down one level of tree and get next element
				of list */
			else
				{
				if(temppointer.next != null)
					{
					nodepointer = nodepointer.down;
					}
				}
			temppointer = temppointer.next;
			}
		}
/* if got to end of list in while loop then exit and return nodepointer */

	if(!finished)
		{
		pointer = nodepointer;
		return(true);
		}

	else
		{
		pointer = null;
		return(false);
		}
}



/*
Parameters:
	tree:         the head node of a tree
  path[]:       an array of arrows
  path-length:  length of path[]
  previous:     node above the last node entered
Purpose:
	To search a tree and determine if path is in the tree,
	if it is not then path is added to the tree.
Design:
	Returns true if path is in the tree and if it is not returns false
	and adds path to the tree.
*/

public boolean search_and_add(nodetype tree, int path[], int path_length, nodetype node, nodetype theprev, Category A, TextArea viewbox)
{
  boolean failed_search;      /* true when finished searching for path in tree
                               - rest of path, if any, to be copied */
  previous = new nodetype();   /*previous is the node above `node'
                                 - used in check_one-one */
  int path_index;               // counts down position in path

	failed_search = false;

  path_index = path_length - 1; //top index in array path[]

  node = tree;
  theprev = tree;
  previous = tree;


  if (tree.down == null)
	/*if tree is empty, set node to top of tree and do not search
	tree - rather add path to tree after skipping while */
  {
		failed_search = true;
	}
  else //there is a possible path in the tree, set node to its start
  {
     node = tree.down;
  }

/*search for path until path is found  (path_index = 0)
             or tail of path is not in tree (finished) */
	while((path_index > -1) && (!failed_search))
	{
    if(!search_level(node,path[path_index]))
		/*  if current element of path not found in its level
       of tree then exit while loop ...*/
	  {
			failed_search = true;
      //previous = node;
      /* ...and make node the parent of level just searched */
      node = node.across;
    }
  	else
		/* so current element of path is in tree ...*/
 	  {
      if(path_index == 0)
      // reached end of path, so path is in tree
			{
          return true;
      }
      else //more elements in path..
      {
			  if(node.down == null)
         // ...and we are at a leaf
			  {
          failed_search = true;  // reached leaf with more nodes in path[]
        }
		    else
		    /* current element of path in tree, and not at leaf
		     of tree so move down one level of tree and get next element
		     of path */
			  {
          theprev = node;
          previous = node;
//          viewbox.appendText("previous.data = " + previous.data + "\n");
					node = node.down;
			  } // if node.down == null
       } // end if path_index = 0
      } // end if not search_level(node, path[path_index])
    path_index--;
  } // end-while path_index > -1 and not finished search

/* entire path not in tree so add rest of path to tree and return false */
	while(path_index > -1)
    {
      theprev = node;
      previous = node;
			node = add_node(node,path[path_index], A, viewbox);
			path_index--;
    }  //end while
	return(false);
}   //end search and add

public void display_tree(nodetype tree, TextArea v, Category A)
{
  nodetype temp;

  temp = tree;
	v.appendText(temp.data + "\n" );
	if (temp.tree != null)
	{
  	v.appendText("X entering subtree\n" );
	  display_tree(temp.tree, v, A);
	}
  //v.appendText(" tree\n");
	if (temp.down != null)
		{
		display_tree(temp.down, v, A);
		}
  //v.appendText("down\n");
  if (temp.flag)
  {
   v.appendText("temp.flag = true\n");
		display_tree(temp.across, v, A);
	}
  //v.appendText("across\n");
}

/*
Parameters:
	tree1, tree2 :  the two trees to be tested for equality
Purpose:
	To compare two trees and return whether or not they are equal trees
Design:
	Traverses both trees and compares each node to see if they are equal
*/

public boolean equal_trees(nodetype tree1, nodetype tree2, TextArea temp)
{
  nodetype temp1 = new nodetype();
  nodetype temp2 = new nodetype();

	temp1 = tree1;
	temp2 = tree2;
	if(temp1.data != temp2.data)
  {
    temp.appendText("returning false from equal trees - unequal data");
		return(false);
	}
	else
  {
		if((temp1.down == null) != (temp2.down == null))
		{
			return(false);
		}
		if ((temp1.down != null) && (temp2.down != null))
		{
			if(!(equal_trees(temp1.down,temp2.down, temp)))
			{
				return(false);
			}
		}
		if((temp1.flag) != (temp2.flag))
		{
			return(false);
		}
  	if((temp1.flag) && (temp2.flag))
		{
    	if(!(equal_trees(temp1.across,temp2.across, temp)))
			{
	  		return(false);
			}
		}
  }
	return(true);
}



/*
Parameters:
	list:a pointer to a pointer to the first element of the list being made
	path:an array of integers to be stored in the list
Purpose:
	To create a list of integers
Design:
	The integers are stored in the list in reverse order. The first integer
	of the array is stored at the end of the list.
*/

void make_list(listnodetype head, int path[], int path_length, TextArea v)
//Path length is acquired by taking path_len(path)
{
  listnodetype temppointer = new listnodetype();
  int i;

	head = null;
	for(i=0; i < path_length; i++)
		{
            if (i==0)
              temppointer.next = null;
            else
    	        temppointer.next = head;
		  temppointer.data = path[i];

		  head = temppointer;
		}
}

void display_list( listnodetype lst, TextArea v)
{
  listnodetype temp;

	temp = lst;
  v.appendText("display_list\n\n");
	while(temp.next != null)
	{
		v.appendText(temp.data + "\n");
		temp = temp.next;
	}
}


}/* end class tree */
